/****************************************************************
**
** Attal : Lords of Doom
**
** decorationEffect.h
** Widget for editing effects of decoration
**
** Version : $Id: decorationEffect.h,v 1.1 2003/08/15 10:41:55 audoux Exp $
**
** Author(s) : Pascal Audoux
**
** Date : 15/08/2003
**
** Licence :
**	This program is free software; you can redistribute it and/or modify
**   	it under the terms of the GNU General Public License as published by
**     	the Free Software Foundation; either version 2, or (at your option)
**      any later version.
**
**	This program is distributed in the hope that it will be useful,
** 	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
****************************************************************/


#ifndef DECORATIONEFFECT_H
#define DECORATIONEFFECT_H


// generic include files
// include files for QT
#include <qdialog.h>
#include <qlistbox.h>
// application specific include files
#include "libCommon/genericDecoration.h"

class QSpinBox;
class QComboBox;

/*              ------------------------------
 *                         DecorationEffect
 *              ------------------------------ */

class DecorationEffect : public QWidget
{
	Q_OBJECT
public:
	DecorationEffect( QWidget * parent = 0, const char * name = 0 );

	void init( GenericDecoration * deco );

	void save();

public slots:
	void slot_add();

	void slot_edit();

	void slot_del();

protected:
	QListBox * _listBox;
	QPushButton * _pbAdd, * _pbEdit, * _pbDel;
	GenericDecoration * _deco;
};

/*              ------------------------------
 *                         EffectItem
 *              ------------------------------ */

class EffectItem : public QListBoxText
{
public:
	EffectItem( GenericDecoration::EffectType type, uint param );

	GenericDecoration::EffectType getType() { return _type; }

	void setType( GenericDecoration::EffectType type ) { _type = type; computeLabel(); }

	uint getParam() { return _param; }

	void setParam( uint param ) { _param = param; computeLabel(); }

	void computeLabel();

protected:
	uint _param;
	GenericDecoration::EffectType _type;
};

/*              ------------------------------
 *                         DialogEffect
 *              ------------------------------ */

class DialogEffect : public QDialog
{
	Q_OBJECT
public:
	DialogEffect( QWidget * parent = 0, const char * name = 0 );

	GenericDecoration::EffectType getType() { return _type; }

	void setType( GenericDecoration::EffectType type ) { _type = type; updateDialog(); }

	uint getParam() { return _param; }

	void setParam( uint param ) { _param = param; updateDialog(); }

	void updateDialog();

public slots:
	void accept();

protected:
	uint _param;
	GenericDecoration::EffectType _type;
	QComboBox * _combo;
	QSpinBox * _spin;

};



#endif // DECORATIONEFFECT_H
